@@ -138,7 +138,7 @@ fits_print_extension_info(struct fitsparams *p)
       switch(hdutype)
         {
         case IMAGE_HDU:
-          gal_fits_img_info(fptr, &type, &ndim, &dsize);
+          gal_fits_img_info(fptr, &type, &ndim, &dsize, NULL, NULL);
           tstr=gal_type_name(type , 1);
           break;
 
@@ -286,7 +286,8 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
       hdu=gal_list_str_pop(&list);
 
       /* Open the FITS file at the specified HDU. */
-      in=gal_fits_hdu_open(p->filename, hdu, READWRITE);
+      in=gal_fits_hdu_open(p->filename, hdu,
+                           cut1_copy0 ? READWRITE : READONLY);
 
       /* Copy to the extension. */
       if( fits_copy_hdu(in, out, 0, &status) )
@@ -312,7 +313,7 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
 int
 fits(struct fitsparams *p)
 {
-  int r=EXIT_SUCCESS, printhduinfo=1;;
+  int r=EXIT_SUCCESS, printhduinfo=1;
 
   switch(p->mode)
     {
