@@ -91,12 +91,12 @@ clumps_oversegment(struct clumps_thread_params *cltprm)
                                   GAL_TYPE_INVALID, 2, checkdsize,
                                   NULL, 0, 0, NULL, NULL, NULL);
   tile->block=p->conv;
-  gal_checkset_check_remove_file(filename, 0, 0);
+  gal_checkset_writable_remove(filename, 0, 0);
   if(p->cp.numthreads!=1)
     error(EXIT_FAILURE, 0, "in the debugging mode of `clumps_oversegment' "
           "only one thread must be used");
   crop=gal_data_copy(tile);
-  gal_fits_img_write(crop, filename, NULL, PROGRAM_STRING);
+  gal_fits_img_write(crop, filename, NULL, PROGRAM_NAME);
   gal_data_free(crop);
   printf("blank: %u\nriver: %u\ntmpcheck: %u\ninit: %u\nmaxlab: %u\n",
          (int32_t)GAL_BLANK_INT32, (int32_t)CLUMPS_RIVER,
@@ -107,6 +107,10 @@ clumps_oversegment(struct clumps_thread_params *cltprm)
   **********************************************/
 
 
+  /* If the size of the indexs is zero, then this function is pointless. */
+  if(indexs->size==0) { cltprm->numinitclumps=0; return; }
+
+
   /* Sort the given indexs based on their flux (`gal_qsort_index_arr' is
      defined as static in `gnuastro/qsort.h') */
   gal_qsort_index_arr=p->conv->array;
@@ -146,10 +150,10 @@ clumps_oversegment(struct clumps_thread_params *cltprm)
 
             /* A small sanity check. */
             if(Q!=NULL || cleanup!=NULL)
-              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we "
-                    "can fix this problem. `Q' and `cleanup' should be NULL "
-                    "but while checking the equal flux regions they aren't",
-                    __func__, PACKAGE_BUGREPORT);
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so "
+                    "we can fix this problem. `Q' and `cleanup' should be "
+                    "NULL but while checking the equal flux regions they "
+                    "aren't", __func__, PACKAGE_BUGREPORT);
 
             /* Add this pixel to a queue. */
             gal_list_sizet_add(&Q, *a);
@@ -344,7 +348,7 @@ clumps_oversegment(struct clumps_thread_params *cltprm)
             crop=gal_data_copy(tile);
             crf=(cr=crop->array)+crop->size;
             do if(*cr==CLUMPS_RIVER) *cr=0; while(++cr<crf);
-            gal_fits_img_write(crop, filename, NULL, PROGRAM_STRING);
+            gal_fits_img_write(crop, filename, NULL, PROGRAM_NAME);
             gal_data_free(crop);
           }
         **********************************************/
@@ -808,6 +812,10 @@ clumps_make_sn_table(struct clumps_thread_params *cltprm)
   size_t i, ind, counter=0, infodsize[2]={tablen, INFO_NCOLS};
 
 
+  /* If there were no initial clumps, then ignore this function. */
+  if(cltprm->numinitclumps==0) { cltprm->sn=NULL; return; }
+
+
   /* Allocate the arrays to keep the final S/N table (and possibly S/N
      index) for this object or tile. */
   cltprm->sn        = &cltprm->clprm->sn[ cltprm->id ];
@@ -920,8 +928,8 @@ clumps_correct_sky_labels_for_check(struct clumps_thread_params *cltprm,
 
   /* A small sanity check. */
   if(gal_tile_block(tile)!=p->clabel)
-    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address the "
-          "problem. `tile->block' must point to the `clabel' dataset",
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address "
+          "the problem. `tile->block' must point to the `clabel' dataset",
           __func__, PACKAGE_BUGREPORT);
 
 
@@ -1001,6 +1009,7 @@ clumps_find_make_sn_table(void *in_prm)
       cltprm.id = tind  = tprm->indexs[i];
       tile = &p->ltl.tiles[tind];
 
+
       /* Change the tile's pointers to the binary image (which has 1 for
          detected pixels and 0 for un-detected regions). */
       tarray=tile->array;
@@ -1008,6 +1017,7 @@ clumps_find_make_sn_table(void *in_prm)
       tile->array = gal_tile_block_relative_to_other(tile, p->binary);
       tile->block = p->binary;
 
+
       /* Get the number of usable elements in this tile (note that tiles
          can have blank pixels), so we can't simply use `tile->size'. */
       if(tile->flag & GAL_DATA_FLAG_HASBLANK)
@@ -1018,12 +1028,14 @@ clumps_find_make_sn_table(void *in_prm)
         }
       else num=tile->size;
 
+
       /* Find the number of detected pixels over this tile. Since this is
          the binary image, this is just the sum of all the pixels. */
       tmp=gal_statistics_sum(tile);
       numdet=*((double *)(tmp->array));
       gal_data_free(tmp);
 
+
       /* See if this tile should be used or not (has enough undetected
          pixels). Note that it might happen that some tiles are fully
          blank. In such cases, it is important to first check the number of
@@ -1043,6 +1055,7 @@ clumps_find_make_sn_table(void *in_prm)
           tile->array = gal_tile_block_relative_to_other(tile, p->clabel);
           tile->block = p->clabel;
 
+
           /* We need to set all the pixels on the edge of the tile to
              rivers and not include them in the list of indexs to set
              clumps. To do that, we need this tile's starting
@@ -1052,6 +1065,7 @@ clumps_find_make_sn_table(void *in_prm)
                                                          p->clabel->type),
                                        ndim, dsize, scoord);
 
+
           /* Add the index of every sky element to the array of
              indexs. Note that since we know the array is always of type
              `int32_t', we can call the `GAL_TILE_PO_OISET' macro to avoid
@@ -1103,30 +1117,37 @@ clumps_find_make_sn_table(void *in_prm)
                 }
             });
 
+
           /* Correct the number of indexs. */
           cltprm.indexs->size=cltprm.indexs->dsize[0]=c;
 
+
           /* Generate the clumps over this region. */
           clumps_oversegment(&cltprm);
 
+
           /* Set all river pixels to CLUMPS_INIT (to be distinguishable
              from the detected regions). */
           GAL_TILE_PO_OISET( int32_t, int, tile, NULL, 0, 1,
                              {if(*i==CLUMPS_RIVER) *i=CLUMPS_INIT;} );
 
+
           /* For a check, the step variable will be set. */
           if(clprm->step==1)
             { gal_data_free(cltprm.indexs); continue; }
 
+
           /* Make the clump S/N table. */
           clumps_make_sn_table(&cltprm);
 
+
           /* If the user wanted to check the steps, remove the clumps that
              weren't used from the `clabel' image (they have been already
              excluded from the table). */
           if(cltprm.snind)
             clumps_correct_sky_labels_for_check(&cltprm, tile);
 
+
           /* Clean up. */
           gal_data_free(cltprm.indexs);
         }
@@ -1242,7 +1263,7 @@ clumps_true_find_sn_thresh(struct noisechiselparams *p)
              demo, set all Sky regions to blank and all clump macro values
              to zero. */
           gal_fits_img_write(p->clabel, p->segmentationname, NULL,
-                             PROGRAM_STRING);
+                             PROGRAM_NAME);
 
           /* Increment the step counter. */
           ++clprm.step;
@@ -1255,10 +1276,11 @@ clumps_true_find_sn_thresh(struct noisechiselparams *p)
   else
     {
       clprm.step=0;
-      gal_threads_spin_off(clumps_find_make_sn_table, &clprm, p->ltl.tottiles,
-                           p->cp.numthreads);
+      gal_threads_spin_off(clumps_find_make_sn_table, &clprm,
+                           p->ltl.tottiles, p->cp.numthreads);
     }
 
+
   /* Destroy the mutex if it was initialized. */
   if( p->cp.numthreads>1 && (p->checksegmentation || p->checkclumpsn) )
     pthread_mutex_destroy(&clprm.labmutex);
@@ -1268,6 +1290,12 @@ clumps_true_find_sn_thresh(struct noisechiselparams *p)
   for(i=0;i<p->ltl.tottiles;++i)
     if(clprm.sn[i].ndim)  /* Only on tiles were an S/N was calculated. */
       numsn+=clprm.sn[i].size;
+  if( numsn < p->minnumfalse )
+    error(EXIT_FAILURE, 0, "only %zu clumps could be identified in the "
+          "undetected regions. This is less than %zu (value to "
+          "`--minnumfalse' option). Please either decrease this value or "
+          "other options to change prior processing steps", numsn,
+          p->minnumfalse);
 
 
   /* Allocate the space to keep all the S/N values. */
@@ -1275,7 +1303,7 @@ clumps_true_find_sn_thresh(struct noisechiselparams *p)
                     p->cp.minmapsize, "CLUMP_S/N", "ratio",
                     "Signal-to-noise ratio");
   snind = ( p->checkclumpsn
-            ? gal_data_alloc(NULL, clprm.snind->type, 1, &numsn, NULL, 0,
+            ? gal_data_alloc(NULL, GAL_TYPE_INT32, 1, &numsn, NULL, 0,
                              p->cp.minmapsize, "CLUMP_ID", "counter",
                              "Unique ID for this clump.")
             : NULL );
@@ -1421,57 +1449,67 @@ clumps_det_keep_true_relabel(struct clumps_thread_params *cltprm)
   size_t ndim=p->input->ndim, *dsize=p->input->dsize;
 
   int istouching;
-  size_t *s, *sf;
-  float *sn=cltprm->sn->array;
-  int32_t *l, *lf, curlab=1, *clabel=p->clabel->array;
-  size_t i, *dinc=gal_dimension_increment(ndim, dsize);
-  int32_t *newlabs=gal_data_malloc_array(GAL_TYPE_INT32,
-                                         cltprm->numinitclumps+1, __func__,
-                                         "newlabs");
+  size_t i, *s, *sf, *dinc;
+  float *sn = cltprm->sn ? cltprm->sn->array : NULL;
+  int32_t *l, *lf, *newlabs, curlab=1, *clabel=p->clabel->array;
 
-  /* Initialize the new labels with CLUMPS_INIT (so the diffuse area can be
-     distinguished from the clumps). */
-  lf=(l=newlabs)+cltprm->numinitclumps+1; do *l++=CLUMPS_INIT; while(l<lf);
-
-
-  /* Set the new labels. Here we will also be removing clumps with a peak
-     that touches a river pixel. */
-  if(p->keepmaxnearriver)
+  /* If there were no clumps over the detection, then just set the number
+     of true clumps to zero, otherwise, see which ones should be
+     removed. */
+  if(cltprm->sn)
     {
-      for(i=1;i<cltprm->numinitclumps+1;++i)
-        if( sn[i] > p->clumpsnthresh ) newlabs[i]=curlab++;
-    }
-  else
-    {
-      for(i=1;i<cltprm->numinitclumps+1;++i)
+      /* Allocate the necessary arrays. */
+      newlabs=gal_data_malloc_array(GAL_TYPE_INT32,
+                                    cltprm->numinitclumps+1, __func__,
+                                    "newlabs");
+      dinc=gal_dimension_increment(ndim, dsize);
+
+      /* Initialize the new labels with CLUMPS_INIT (so the diffuse area
+         can be distinguished from the clumps). */
+      lf=(l=newlabs)+cltprm->numinitclumps+1;
+      do *l++=CLUMPS_INIT; while(l<lf);
+
+      /* Set the new labels. Here we will also be removing clumps with a peak
+         that touches a river pixel. */
+      if(p->keepmaxnearriver)
         {
-          /* Check if all the neighbors of this top element are touching a
-             river or not. */
-          istouching=0;
-          GAL_DIMENSION_NEIGHBOR_OP(cltprm->topinds[i], ndim, dsize, ndim,
-                                    dinc,
-                                    { if(clabel[nind]==0) istouching=1; });
-
-          /* If the peak isn't touching a river, then check its S/N and if
-             that is also good, give it a new label. */
-          if( !istouching && sn[i] > p->clumpsnthresh ) newlabs[i]=curlab++;
+          for(i=1;i<cltprm->numinitclumps+1;++i)
+            if( sn[i] > p->clumpsnthresh ) newlabs[i]=curlab++;
         }
+      else
+        {
+          for(i=1;i<cltprm->numinitclumps+1;++i)
+            {
+              /* Check if all the neighbors of this top element are
+                 touching a river or not. */
+              istouching=0;
+              GAL_DIMENSION_NEIGHBOR_OP(cltprm->topinds[i], ndim, dsize,
+                                        ndim, dinc,
+                                        {
+                                          if(clabel[nind]==0)
+                                            istouching=1;
+                                        });
+
+              /* If the peak isn't touching a river, then check its S/N and
+                 if that is also good, give it a new label. */
+              if( !istouching && sn[i] > p->clumpsnthresh )
+                newlabs[i]=curlab++;
+            }
+        }
+
+      /* Correct the clump labels. Note that the non-clumpy regions over the
+         detections (rivers) have already been initialized to CLUMPS_INIT
+         (which is negative). So we'll just need to correct the ones with a
+         value larger than 0. */
+      sf=(s=cltprm->indexs->array)+cltprm->indexs->size;
+      do if(clabel[*s]>0) clabel[*s] = newlabs[ clabel[*s] ]; while(++s<sf);
+
+      /* Save the total number of true clumps in this detection. */
+      cltprm->numtrueclumps=curlab-1;
+
+      /* Clean up. */
+      free(dinc);
+      free(newlabs);
     }
-
-
-  /* Save the total number of true clumps in this detection. */
-  cltprm->numtrueclumps=curlab-1;
-
-
-  /* Correct the clump labels. Note that the non-clumpy regions over the
-     detections (rivers) have already been initialized to CLUMPS_INIT
-     (which is negative). So we'll just need to correct the ones with a
-     value larger than 0. */
-  sf=(s=cltprm->indexs->array)+cltprm->indexs->size;
-  do if(clabel[*s]>0) clabel[*s] = newlabs[ clabel[*s] ]; while(++s<sf);
-
-
-  /* Clean up. */
-  free(dinc);
-  free(newlabs);
+  else cltprm->numtrueclumps=0;
 }
