@@ -1,5 +1,5 @@
 /* Test suite for argp.
-   Copyright (C) 2006-2007, 2009-2016 Free Software Foundation, Inc.
+   Copyright (C) 2006-2007, 2009-2017 Free Software Foundation, Inc.
    This file is part of the GNUlib Library.
 
    This program is free software: you can redistribute it and/or modify
@@ -26,6 +26,8 @@
 # include <strings.h>
 #endif
 
+#include "macros.h"
+
 struct test_args
 {
   int test;
@@ -63,7 +65,7 @@ group1_parser (int key, char *arg, struct argp_state *state)
 
     case 'r':
       args->read = 1;
-      /* fall through */
+      FALLTHROUGH;
     case 'f':
       args->file = arg;
       break;
