@@ -34,7 +34,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Configuration macros: */
 
-#define GAL_CONFIG_VERSION              "0.3.7-a629"
+#define GAL_CONFIG_VERSION              "0.4"
 
 #define GAL_CONFIG_HAVE_LIBGIT2         1
 
@@ -53,6 +53,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #define GAL_CONFIG_BIN_OP_FLOAT32       1
 #define GAL_CONFIG_BIN_OP_FLOAT64       1
 
+#define GAL_CONFIG_SIZEOF_LONG          8
 #define GAL_CONFIG_SIZEOF_SIZE_T        8
 
 
