@@ -55,4 +55,4 @@ if [ ! -f $cat      ]; then echo "$cat does not exist.";   exit 77; fi
 
 # Actual test script
 # ==================
-$execname $cat --naxis1=100 --naxis2=100 --crpix1=-99 --individual
+$execname $cat --naxis=100,100 --crpix=-99,1 --individual
